<?php

declare(strict_types=1);

namespace Drupal\composable_editor\Form;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The Composable SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Construct a new Settings form.
   *
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   *   Drupal entity type bundle service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   Drupal entity field manager.
   */
  public function __construct(
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.bundle.info'),
      $container->get('entity_field.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'composable_edit_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['composable_editor.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['basic'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Basic settings:'),
      '#name' => 'basic',
    ];

    $form['basic']['framework'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Framework module name'),
      '#default_value' => $this->config('composable_editor.settings')->get('framework'),
      '#group' => 'basic',
    ];

    $form['basic']['disable_styles'] = [
      '#type' => 'checkbox',
      '#title' => 'Disable default Off-Canvas Styles',
      '#default_value' => boolval($this->config('composable_editor.settings')->get('disable_styles')),
      '#group' => 'basic',
    ];

    $form['basic']['open_load'] = [
      '#type' => 'checkbox',
      '#title' => 'Open Off-Canvas on page load',
      '#default_value' => boolval($this->config('composable_editor.settings')->get('open_load')),
      '#group' => 'basic',
    ];

    $isFieldGroup = \Drupal::moduleHandler()->moduleExists('field_group');

    $form['wrapper'] = [
      '#type' => 'details',
      '#title' => $this->t('Components Field Group wrapper:'),
      '#name' => 'wrapper',
      '#open' => $isFieldGroup,
      '#description' => 'Make sure you install and enable the Field Group module first to continue configuration.',
    ];

    if ($isFieldGroup) {
      $form['wrapper']['wrapper_type'] = [
        '#type' => 'select',
        '#title' => 'Select wrapper type',
        '#options' => [
          'none' => 'None',
          'tab' => 'Tabs',
        ],
        '#default_value' => $this->config('composable_editor.settings')->get('wrapper.type'),
        '#group' => 'wrapper',
      ];

      $form['wrapper']['wrapper_mode'] = [
        '#type' => 'select',
        '#title' => 'Select wrapper mode',
        '#options' => [
          'horizontal' => 'Horizontal',
        ],
        '#default_value' => $this->config('composable_editor.settings')->get('wrapper.mode'),
        '#states' => [
          'visible' => [
            [':input[name="wrapper_type"]' => ['!value' => 'none']],
          ],
        ],
        '#group' => 'wrapper',
      ];

      $form['wrapper']['wrapper_selector'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Wrapper selector'),
        '#default_value' => $this->config('composable_editor.settings')->get('wrapper.selector'),
        '#group' => 'wrapper',
        '#states' => [
          'visible' => [
            [':input[name="wrapper_type"]' => ['!value' => 'none']],
          ],
        ],
      ];

      $form['wrapper']['wrapper_position'] = [
        '#type' => 'number',
        '#title' => $this->t('Wrapper position'),
        '#default_value' => $this->config('composable_editor.settings')->get('wrapper.position'),
        '#group' => 'wrapper',
        '#states' => [
          'visible' => [
            [':input[name="wrapper_type"]' => ['!value' => 'none']],
          ],
        ],
        '#description' => 'The position for components 1 is the first element.',
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('composable_editor.settings')
      ->set('framework', $form_state->getValue('framework'))
      ->set('disable_styles', boolval($form_state->getValue('disable_styles')))
      ->set('open_load', boolval($form_state->getValue('open_load')))
      ->set('wrapper.type', $form_state->getValue('wrapper_type'))
      ->set('wrapper.mode', $form_state->getValue('wrapper_mode'))
      ->set('wrapper.selector', $form_state->getValue('wrapper_selector'))
      ->set('wrapper.position', $form_state->getValue('wrapper_position'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
